/******************************************************************************
File Name : mx_dio_control.c

Description :
	This utility can control DO HIGH/LOW signal, and get DI/DO HIGH/LOW signal.

Usage :
	1.Get signal:
		./mx_dio_control -g [val] -n [slot]
			val=0 -> DO, val=1 -> DI
			slot -> slot number
	2.Set signal
		./mx_dio_control -s [val] -n [slot]
			val=0 -> LOW, val=1 -> HIGH
			slot -> slot number

History :
	Versoin		Author		Date		Comment
	1.0		HarryYJ.Jhou	07-22-2015	First Version
	1.1		EthanSH Lee	02-16-2017	Separate binary and config
*******************************************************************************/

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<unistd.h>
#include	"moxadevice.h"
#include	<fcntl.h>

#define GET 0
#define SET 1
#define DO 0
#define DI 1
#define GET 0
#define SET 1
#define MAX_GPIO_VALUE 200
#define MAX_CONFIG_VARIABLE_LEN 20
#define CONFIG_LINE_BUFFER_SIZE 100
#define CONFIG "dio.conf"
#define GPIO_EXPORT "/sys/class/gpio/export"

char	*DataString[2]={"Low  ", "High "};

usage(){
	printf("Moxa DIO get/set utility.\n");
	printf("Usage: mx_dio_control [-g|-s] -n #slot\n");
	printf("\t-g [1|0] Get DI/DO value\n");
	printf("\t         0:DO\n");
	printf("\t         1:DI\n");
	printf("\t-s [1|0] Set DO value\n");
	printf("\t         0:LOW\n");
	printf("\t         1:HIGH\n");
	printf("\t-n [#num] port number\n");
	printf("\t         num: port number DI:1~%d DO:1~%d\n", config.max_din_port, config.max_dout_port);
	printf("\n");
	printf("Example:\n");
	printf("\tGet port 2 DI value\n");
	printf("\t         mx_dio_control -g 1 -n 2\n");
	printf("\tSet port 2 DO value to LOW\n");
	printf("\t         mx_dio_control -s 0 -n 2\n");
	exit(-1);
}

int get_dio_gpio(int gpio_num){
	//char GPIO_PATH[]="/home/moxa/dio-0221/gpio/gpio";
	char GPIO_PATH[]="/sys/class/gpio/gpio";
	char str_num[5];
	char value[]="/value";
	char GPIO_VALUE[MAX_GPIO_VALUE];
	int ret;

	sprintf(str_num, "%d", gpio_num);
	
	strcpy(GPIO_VALUE, GPIO_PATH);
	strcat(GPIO_VALUE, str_num);
	strcat(GPIO_VALUE, value);
	//printf("GPIO_VALUE=%s\n", GPIO_VALUE);

	FILE *file=fopen(GPIO_VALUE,"r");
	if (file) {
        	fscanf(file, "%d", &ret);
	}
	fclose(file);
	return ret;
}

void write_file(char *file ,char *str){
	FILE *fp=fopen(file,"w");
	int ret;
	//printf("file=%s\n", file);
	if (fp == NULL)
	{	
		printf ("Fail to open %s\nProgram exits.\n",file);
		exit(1);
	}
	
	ret = fprintf(fp, str);
	//printf("str=%s ,ret=%d\n", str, ret);
	fclose(fp);
}

void set_dout_gpio (int gpio_num, int low_high){
	//char GPIO_PATH[]="/home/moxa/dio-0221/gpio/gpio";
	char GPIO_PATH[]="/sys/class/gpio/gpio";
	char str_num[5];
	char low_high_str[1];
	char direction[]="/direction";
	char value[]="/value";
	char GPIO_DIRECTION[MAX_GPIO_VALUE];
	char GPIO_VALUE[MAX_GPIO_VALUE];
	FILE *fp;

	sprintf(low_high_str, "%d", low_high);
	sprintf(str_num, "%d", gpio_num);
	//char *str_number=&str_num[0];
	//printf("%s ", str_number);
	strcpy(GPIO_DIRECTION, GPIO_PATH);
	strcat(GPIO_DIRECTION, str_num);
	strcat(GPIO_DIRECTION, direction); 
	//printf("GPIO_DIRECTION=%s\n", GPIO_DIRECTION);
	
	strcpy(GPIO_VALUE, GPIO_PATH);
	strcat(GPIO_VALUE, str_num);
	strcat(GPIO_VALUE, value);
	//printf("GPIO_VALUE=%s\n", GPIO_VALUE);
	//if (!(fp = fopen(GPIO_DIRECTION, "w"))){
 		write_file(GPIO_EXPORT, str_num);      		 
 		write_file(GPIO_DIRECTION,"out");      		 
		//printf("The File %s\t not Found\n", GPIO_DIRECTION);
    	//}
	//fclose(fp);
	write_file(GPIO_VALUE, low_high_str);
}

int *read_gpio_from_config_line(char* config_line , int array_size) {    
		char prm_name[MAX_CONFIG_VARIABLE_LEN];
		char prm_value[MAX_CONFIG_VARIABLE_LEN];
		int i=0;
		int *gpio_array;
	
    		sscanf(config_line, "%s\t%s\n", prm_name, prm_value);
		gpio_array = (int*)malloc(sizeof(int) * array_size);
		char *number=strtok(prm_value, ",");
		
		while(number != NULL){
			*(gpio_array + i) = atoi(number);
			number = strtok(NULL, ",");
			i++;
		}
		//printf("gpio_array=%d\n", *(gpio_array+0) );	
		return gpio_array;
}

int read_int_from_config_line(char* config_line) {    
		char prm_name[MAX_CONFIG_VARIABLE_LEN];
    		int val;
    		sscanf(config_line, "%s\t%d\n", prm_name, &val);
    		return val;
}

void read_str_from_config_line(char* config_line, char* val) {    
  		char prm_name[MAX_CONFIG_VARIABLE_LEN];
    		sscanf(config_line, "%s\t%s\n", prm_name, val);
}


void read_config_file(char* config_filename,char* target_product) {
    FILE *fp;
    char buf[CONFIG_LINE_BUFFER_SIZE];
    char *line;
    int flag=0;		
    int count=0;
    char tmp_key[MAX_CONFIG_VARIABLE_LEN];
    char tmp_value[MAX_CONFIG_VARIABLE_LEN];
	
    if ((fp=fopen(config_filename, "r")) == NULL) {
        fprintf(stderr, "Failed to open config file %s\n", config_filename);
        exit(EXIT_FAILURE);
    }

    while(! feof(fp)) {
     	fgets(buf, CONFIG_LINE_BUFFER_SIZE, fp);
		
	if (buf[0] == '#') 
		continue;
	
	if (strstr(buf, "PRODUCT")) {
		read_str_from_config_line(buf, tmp_key);	
		if( strcmp(tmp_key, target_product) == 0 ){
			flag=1; //turn on record flag
			strcpy(config.product , tmp_key);
			continue; //skip this round of while
    		}
	}
	if (flag == 1){
			char pre_name[MAX_CONFIG_VARIABLE_LEN];
			char pre_value[MAX_CONFIG_VARIABLE_LEN];
			int val;
			switch (count) {
            			case 0:
                		config.max_din_port=read_int_from_config_line(buf);
				count++;
                		break;

				case 1:
                		config.max_dout_port=read_int_from_config_line(buf);
				count++;
				break;
				
				case 2:
                		read_str_from_config_line(buf, config.method);
				count++;
				break;

            			case 3:
				if( strcmp (config.method, "ioctl") == 0 ){
                			read_str_from_config_line(buf, config.dio_node_name);
					//printf("dio_node_name=%s\n",config.dio_node_name);
					flag=0;
				}else if ( strcmp (config.method, "gpio") == 0 ){
					config.dout_gpio = read_gpio_from_config_line(buf, config.max_dout_port);
					//printf("config.dout_gpio=%d %d %d\n", *(config.dout_gpio+0), *(config.dout_gpio+1), *(config.dout_gpio+2));
					count++;
					//flag=0;
					//printf("din_port[0]=%d %d %d\n", *(config.din_gpio), *(config.din_gpio+1) , *(config.din_gpio+2));
				}else{
					printf("Error when parsing %s at METHOD.", CONFIG);
					exit(-1);
                		}
				break;
	
				case 4:
				if( strcmp (config.method, "gpio") == 0 ){
					config.din_gpio = read_gpio_from_config_line( buf, config.max_din_port);
					//printf("config.din_gpio= %d %d %d\n", *(config.din_gpio+0), *(config.din_gpio+1), *(config.din_gpio+2));
					flag=0; //turn off record flag
				}
				break;
        		}
	}	
    }
    fclose(fp);	
}

int main(int argc, char * argv[])
{
	int 	i, j, state, retval;
	int	c, val, gs=-1, slot=-1; 
	// val==0 --> DO val==1 --> DI, when gs=GET
	// val==0 --> low, val==1 --> high, when gs=SET
	// gs(get/set)

	char optstring[] = "g:s:n:";
	while ((c = getopt(argc, argv, optstring)) != -1)
		switch (c) {
		case 'g':
			if ( gs == -1 ){
				gs = GET;
				val = atoi(optarg);
			}else{
				usage();
			}
			break;
		case 's':
			if ( gs == -1 ){
				gs = SET;
				val = atoi(optarg);
			}else{
				usage();
			}
			break;
		case 'n':
			slot = atoi(optarg);
			break;
		default:
			usage();
			return -1;
		}
	//printf("gs=%d val=%d slot=%d\n", gs ,val,slot);
	char kversion[100];
	FILE *fp = popen("kversion | awk '{print $1}'", "r");
	fscanf(fp, "%s", &kversion);
	pclose(fp);	
	//printf("kverison=%s\n", kversion);
	
	read_config_file(CONFIG, kversion );
	//printf("product=%s\n",config.product);
	//printf("dio_node=%s\n",config.dio_node_name);
	//printf("max_in_port=%d\n",config.max_din_port);
	//printf("max_out_port=%d\n",config.max_dout_port);
	//printf("method=%s\n",config.method);

	if(strcmp(config.method, "ioctl") == 0)
	{
		create_din_event_array();
		if ( !( ((gs == GET && slot>=1 && ((val == DI && slot<=config.max_din_port) || (val == DO && slot<=config.max_dout_port))))
			|| (gs == SET && slot>=1 && slot<=config.max_dout_port) ) )
			usage();

		if ( gs == GET ){
			if ( val == DI ){
				get_din_state(slot-1, &state);
				printf("DIN(%d):%s\n", slot, DataString[state]);
			}else if ( val == DO ){
				get_dout_state(slot-1, &state);
				printf("DOUT(%d):%s\n", slot, DataString[state]);
			}
		}else if ( gs == SET ){
				retval=set_dout_state(slot-1, val);
				if ( retval ){
					printf("SET DO error\n");
					return -1;
				}
		}
		clear_din_event_array();
	}else if (strcmp(config.method, "gpio") == 0) {
		if ( !( ((gs == GET && slot>=1 && ((val == DI && slot<=config.max_din_port) || (val == DO && slot<=config.max_dout_port))))
			|| (gs == SET && slot>=1 && slot<=config.max_dout_port) ) )
			usage();

		if ( gs == GET ){
			if ( val == DI ){
				int ret_val= get_dio_gpio(*(config.din_gpio+(slot-1)));
				printf("DIN(%d):%s\n", slot, DataString[ret_val]);
				free(config.din_gpio);
			}else if ( val == DO ){
				int ret_val= get_dio_gpio(*(config.dout_gpio+(slot-1)));
				printf("DOUT(%d):%s\n", slot, DataString[ret_val]);
				free(config.dout_gpio);
			}
		}else if ( gs == SET ){
			set_dout_gpio( *(config.dout_gpio+(slot-1)), val);
		}
	}else{
		printf("Error METHOD in %s\n", CONFIG);	
	}
	return 0;
}
